'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _superagent = require('superagent');

var _superagent2 = _interopRequireDefault(_superagent);

var _helpers = require('./helpers');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// HTTP request methods
var methods = ['get', 'post', 'put', 'patch', 'delete', 'head', 'options'];

var http_client = {};

// Define HTTP methods on `http_client` object
var _iteratorNormalCompletion = true;
var _didIteratorError = false;
var _iteratorError = undefined;

try {
	var _loop = function _loop() {
		var method = _step.value;

		http_client[method] = function (destination, data) {
			var options = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

			var url = format_url(destination);

			return new _promise2.default(function (resolve, reject) {
				// Create Http request
				var request = _superagent2.default[method](url);

				// Attach data to the outgoing HTTP request
				if (data) {
					switch (method) {
						case 'get':
							request.query(data);
							break;

						case 'post':
						case 'put':
						case 'patch':
						case 'head':
						case 'options':
							request.send(data);
							break;

						case 'delete':
							throw new Error('"data" supplied for HTTP DELETE request: ' + (0, _stringify2.default)(data));

						default:
							throw new Error('Unknown HTTP method: ' + method);
					}
				}

				// Apply this HTTP request specific HTTP headers
				if (options.headers) {
					request.set(options.headers);
				}

				// Send HTTP request
				request.end(function (error, response) {
					// If HTTP response was received,
					// and if that HTTP response is a JSON object,
					// then the error is the `error` property of that object.
					if (!error && response) {
						error = response.error;
					}

					// If the HTTP request failed, or returned an `error` property
					if (error) {
						// superagent would have already output the error to console
						// console.error(error.stack)

						console.log('[http client] (http request error)');

						// If the 
						if (response) {
							error.code = response.status;

							var content_type = response.get('content-type').split(';')[0].trim();

							if (content_type === 'text/plain') {
								error.message = response.text;
							} else if (content_type === 'text/html') {
								error.html = response.text;
							}
						}

						return reject(error);
					}

					// 204 - No content (e.g. PUT, DELETE)
					if (response.statusCode === 204) {
						return resolve();
					}

					resolve(parse_dates(response.body));
				});
			});
		};
	};

	for (var _iterator = (0, _getIterator3.default)(methods), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
		_loop();
	}
} catch (err) {
	_didIteratorError = true;
	_iteratorError = err;
} finally {
	try {
		if (!_iteratorNormalCompletion && _iterator.return) {
			_iterator.return();
		}
	} finally {
		if (_didIteratorError) {
			throw _iteratorError;
		}
	}
}

function format_url(destination) {
	if ((0, _helpers.is_object)(destination)) {
		// Prepend host and port of the API server to the path.
		return 'http://' + destination.host + ':' + destination.port + destination.path;
	}

	// Prepend prefix to relative URL, to proxy to API server.
	return destination;
}

// JSON date deserializer.
//
// Automatically converts ISO serialized `Date`s
// in JSON responses for Ajax HTTP requests.
//
// Without it the developer would have to convert
// `Date` strings to `Date`s in Ajax HTTP responses manually.
//
// Use as the second, 'reviver' argument to `JSON.parse`: `JSON.parse(json, JSON.date_parser)`
//
// http://stackoverflow.com/questions/14488745/javascript-json-date-deserialization/23691273#23691273

// ISO 8601 date regular expression
var ISO = /^(\d{4})-(\d{2})-(\d{2})T(\d{2}):(\d{2}):(\d{2}(?:\.\d*))(?:Z|(\+|-)([\d|:]*))?$/;

// Walks JSON object tree
function parse_dates(object) {
	// If an array is encountered, 
	// proceed recursively with each element of this array.
	if (object instanceof Array) {
		var i = 0;
		while (i < object.length) {
			object[i] = parse_dates(object[i]);
			i++;
		}
	}
	// If a child JSON object is encountered,
	// convert all of its `Date` string values to `Date`s,
	// and proceed recursively for all of its properties.
	else if ((0, _helpers.is_object)(object)) {
			var _iteratorNormalCompletion2 = true;
			var _didIteratorError2 = false;
			var _iteratorError2 = undefined;

			try {
				for (var _iterator2 = (0, _getIterator3.default)((0, _keys2.default)(object)), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
					var key = _step2.value;

					var value = object[key];
					if (typeof value === 'string' && ISO.test(value)) {
						object[key] = new Date(value);
					} else {
						// proceed recursively
						parse_dates(value);
					}
				}
			} catch (err) {
				_didIteratorError2 = true;
				_iteratorError2 = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion2 && _iterator2.return) {
						_iterator2.return();
					}
				} finally {
					if (_didIteratorError2) {
						throw _iteratorError2;
					}
				}
			}
		}

	// Dates have been converted for this JSON object
	return object;
}

exports.default = http_client;
module.exports = exports['default'];
//# sourceMappingURL=http.js.map